@@ -2,7 +2,7 @@ package Apache::Constants;
 
 use vars qw($VERSION @ISA @EXPORT);
 
-$VERSION = "1.03";
+$VERSION = "1.05";
 
 use Carp ();
 use Exporter ();
@@ -34,7 +34,7 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT);
 			 BAD_GATEWAY 
 			 HTTP_SERVICE_UNAVAILABLE
 			 HTTP_VARIANT_ALSO_VARIES)],
-    common => [qw(OK DECLINED NOT_FOUND FORBIDDEN AUTH_REQUIRED SERVER_ERROR)],
+    common => [qw(OK DECLINED DONE NOT_FOUND FORBIDDEN AUTH_REQUIRED SERVER_ERROR)],
 );
 
 @EXPORT_OK = qw(
@@ -53,12 +53,8 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT);
    BAD_REQUEST
    DECLINED
    DOCUMENT_FOLLOWS
-   DYNAMIC_MODULE_LIMIT
+   DONE
    FORBIDDEN
-   HUGE_STRING_LEN
-   MAX_HEADERS
-   MAX_STRING_LEN
-   METHODS
    M_CONNECT
    M_DELETE
    M_GET
@@ -96,11 +92,18 @@ sub AUTH_REQUIRED () {401}
 	push @export, grep {!$SEEN{$_}++} @$val;
     }
     push @export, grep {!$SEEN{$_}++} @EXPORT;
-    bootstrap Apache::Constants $VERSION;
-    foreach $key (@export) {
-	next if defined &$key;
-	$val = constant($key);
-	eval "sub $key () { $val }";
+    eval { bootstrap Apache::Constants $VERSION; };
+    if($@) {
+	my $gw = $ENV{GATEWAY_INTERFACE} || '';
+	die "$@\n" if substr($gw,0,8) eq 'CGI-Perl';
+	warn "warning: can't `bootstrap Apache::Constants $VERSION' outside of httpd\n";
+    }
+    else {
+	foreach $key (@export) {
+	    next if defined &$key;
+	    $val = constant($key);
+	    eval "sub $key () { $val }";
+	}
     }
 }
 
