@@ -7,8 +7,8 @@ use File::Basename qw(dirname);
 use Cwd qw(fastcwd);
 
 use vars qw($VERSION $Debug);
-#$Id: Registry.pm,v 1.32 1997/06/12 00:17:41 dougm Exp $
-$VERSION = (qw$Revision: 1.32 $)[1];
+#$Id: Registry.pm,v 1.35 1997/07/29 02:11:12 dougm Exp $
+$VERSION = (qw$Revision: 1.35 $)[1];
 
 $Debug ||= 0;
 # 1 => log recompile in errorlog
@@ -17,6 +17,7 @@ $Debug ||= 0;
 Apache->module('Apache::Debug') if $Debug;
 
 my(@cleanup);
+my $Is_Win32 = $^O eq "MSWin32";
 
 sub handler {
     my($r) = @_;
@@ -36,7 +37,7 @@ sub handler {
 	    $r->log_reason("attempt to invoke directory as script", $filename);
 	    return FORBIDDEN;
 	}
-	unless (-x _) {
+	unless ($Is_Win32 or -x _) {
 	    $r->log_reason("file permissions deny server execution",
 			   $filename);
 	    return FORBIDDEN;
@@ -66,6 +67,10 @@ sub handler {
 	$r->log_error("Apache::Registry::handler package $package")
 	   if $Debug & 4;
 
+
+	my $cwd = fastcwd;
+	chdir dirname $r->filename;
+
 	if (
 	    defined($Apache::Registry->{$package}{mtime})
 	    &&
@@ -98,12 +103,7 @@ sub handler {
 			    $sub,
 			    "\n}", # last line comment without newline?
 			   );
-	    {
-		# hide our variables within this block
-                my($r,$filename,$script_name,$mtime,$package,$sub);
-		Apache->untaint($eval);
-                eval $eval;
-            }
+	    compile($eval);
 	    if ($@) {
 		$r->log_error($@);
 		return SERVER_ERROR unless $Debug & 2;
@@ -114,10 +114,11 @@ sub handler {
 	    $Apache::Registry->{$package}{mtime} = $mtime;
 	}
 
-	my $cwd = fastcwd;
-	chdir dirname $r->filename;
 	eval {$package->handler;};
-	chdir $cwd;
+	{
+	    local $^W = 0; #shutup Cwd.pm
+	    chdir $cwd;
+	}
 	$^W = $oldwarn;
 
 	my $err = $@;
@@ -136,13 +137,19 @@ sub handler {
 	for (@cleanup) { &{$_}($r) }
 	@cleanup = ();
 
-	return OK;
+	return $r->status;
     } else {
 	return NOT_FOUND unless $Debug & 2;
 	return Apache::Debug::dump($r, NOT_FOUND);
     }
 }
 
+sub compile {
+    my $eval = shift;
+    Apache->untaint($eval);
+    eval $eval;
+}
+
 #XXX not good enough yet
 my(%switches) = (
    'T' => sub {
@@ -179,7 +186,7 @@ sub push_cleanup {
 }
 
 #trick so we show up under CPAN/modules/by-module/CGI/
-package CGI::Apache;
+package CGI::mod_perl;
 
 1;
 
@@ -199,6 +206,7 @@ Apache::Registry - Run unaltered CGI scrips under mod_perl
  <Location /perl>
  SetHandler perl-script
  PerlHandler Apache::Registry
+ Options ExecCGI 
  ...
  </Directory>
 
