@@ -1,5 +1,8 @@
 package Apache::Status;
 use strict;
+use vars qw($VERSION);
+
+$VERSION = (qw$Revision: 1.13 $)[1];
 
 my(%status) = (
    inc => "Loaded Modules",
@@ -23,12 +26,19 @@ sub handler {
     my $qs = $r->args;
     my $sub = "status_$qs";
     no strict 'refs';
-    $r->print(header());
+    header($r);
     if(defined &$sub) {
-	require CGI::Switch;
+	$r->module('CGI::Switch');
+	$r->module('Devel::Symdump');
+        eval { Exporter::require_version('Devel::Symdump', 2.00); };
+	if($@) {
+	    $r->print("You need to install Devel::Symdump version 2.00 or higher!<p>\n$@");
+	    return 1;
+	}		      
 	$r->print(@{ &{$sub}($r,CGI::Switch->new) });
     }
     elsif (exists $Apache::Registry->{$qs}) { 
+	$r->module('Devel::Symdump');
 	$r->print(symdump($qs));
     }
     else {
@@ -43,9 +53,12 @@ sub handler {
 }
 
 sub header {
+    my $r = shift;
     my $start = scalar localtime $^T;    
     my $srv = Apache::Constants::SERVER_VERSION();
-    return <<"EOF";
+    $r->content_type("text/html");
+    $r->send_http_header;
+    $r->print(<<"EOF");
 <html>
 <head><title>Apache::Status</title></head>
 <body>
@@ -57,7 +70,6 @@ EOF
 
 sub symdump {
     my($package) = @_;
-    require Devel::Symdump;
     my $sob = Devel::Symdump->rnew($package);
     return $sob->as_HTML($package);
 }
